"""
TODO
"""

import tw2.core as twc
from tw2.core.resources import encoder
from tw2.protovis.core import resources as res
from tw2.protovis.core import pvobjects as pvo
import uuid

pv = pvo.PVObjects()

class PVMark(twc.Widget):
    template = "mako:tw2.protovis.core.templates.mark"
    resources = [res.pv_js]

    _initialized = twc.Variable('(bool)', default=False)
    _pv_prop_funcs = twc.Variable('(list) of JSSymbols', default=[])
    _adds = twc.Variable(default=[])
    pvcls = twc.Param()
    parent_js_id = twc.Variable(default=None)
    js_id = twc.Variable(default=None)
    add_method = twc.Variable(default=None)

    def init(self):
        self._pv_prop_funcs = []
        self._initialized = True
        self._adds = []
        return self

    def setupRootPanel(self):
        """ setup root panel... typically done at the start of .prepare """
        self.init().width(self.p_width).height(self.p_height) \
                   .bottom(self.p_bottom).top(self.p_top) \
                   .left(self.p_left).right(self.p_right)

    def hFuncClosure(self, name):
        class NameExtensionWrapper(PVMark):
            mrk, pre = twc.Variable(), twc.Variable()
            def __getattr__(self, name):
                return self.mrk.hFuncClosure('%s.%s' % (self.pre, name))

        # Special exceptions.  These aren't protovis methods.  Just properties
        if name in ['label', 'layer', 'link', 'node', '_parent']:
            if name in ['_parent']:
                name = name[1:]
            return NameExtensionWrapper(mrk=self, pre=name).req().init()

        def handlerFunction(*args, **kwargs):
            if not self._initialized:
                raise ValueError, "panel not initialized.  call .init() first"
            if kwargs:
                raise ValueError, "keyword arguments are disallowed"

            if name.endswith('add'):
                js_id = "%s_%s" % (
                    args[0].src.replace('.',''),
                    str(uuid.uuid4()).replace('-','')
                )
                m = PVMark(
                    pvcls=args[0],
                    js_id=js_id,
                    parent_js_id=self.js_id,
                    add_method=name,
                ).req().init()
                self._adds.append(m)
                return m

            src=".%s(%s)" % (name, encoder.encode(args)[1:-1])

            if name.endswith('anchor'):
                return NameExtensionWrapper(mrk=self, pre=src[1:]).req().init()
               
            f = twc.JSSymbol(src=src)
            if f not in self._pv_prop_funcs:
                self._pv_prop_funcs.append(f)
            
            return self

        return handlerFunction

    def __getattr__(self, name):
        return self.hFuncClosure(name)

class PVWidget(PVMark):
    template = "mako:tw2.protovis.core.templates.widget"

    p_data = twc.Param('(list) data for the widget')
    p_width = twc.Param('The width of the panel, in pixel.', default=400)
    p_height = twc.Param('The height of the panel, in pixels.', default=200)
    p_bottom = twc.Param('The bottom margin, in pixels.', default=20)
    p_top = twc.Param('The top margin, in pixels.', default=5)
    p_left = twc.Param('The left margin, in pixels.', default=20)
    p_right = twc.Param('The right margin, in pixels.', default=10)

    init_js = twc.Param('JSSymbol', default=twc.JSSymbol(src=''))
    pvcls = pv.Panel
    parent_js_id = None
    js_id = 'vis'
    add_method = 'add'



"""
This package contains the basic core protovis resources to be
used by the widgets packaged in the following:

    - tw2.protovis.conventional
    - tw2.protovis.custom
    - tw2.protovis.hierarchies

Get this source from http://github.com/toscawidgets/tw2.protovis.core

Protovis itself can be found here:  http://vis.stanford.edu/protovis/

**Warning**: Protovis itself is deprecated and will be replaced by a new
             javascript library called ``d3``.  You should check out
             ``tw2.d3`` as a replacement.
"""

from widgets import *



""" Samples of how to use tw2.protovis.core

Each class exposed in the widgets submodule has an accompanying Demo<class>
widget here with some parameters filled out.

The demos implemented here are what is displayed in the tw2.devtools
WidgetBrowser.
"""
from widgets import PVWidget, pv
from tw2.core import JSSymbol

import random


class js(JSSymbol):
    def __init__(self, src):
        super(js, self).__init__(src=src)


class DemoPVWidget(PVWidget):
    def __init__(self, *args, **kwargs):
        self.p_data = [
        [random.random() + 0.1 for j in range(4)] for i in range(3)]
        super(DemoPVWidget, self).__init__(*args, **kwargs)
        # Sizing and scales.
        self.init_js = js(
            """
            var data = %s;
            var n = data.length;
            var m = data[0].length;
            var w = %i,
                h = %i,
                x = pv.Scale.linear(0, 1.1).range(0, w),
                y = pv.Scale.ordinal(pv.range(n)).splitBanded(0, h, 4/5);
            """ % (self.p_data, self.p_width, self.p_height))

        self.setupRootPanel()

        # The bars.
        bar = self.add(pv.Panel) \
            .data(js('data')) \
            .top(js('function() y(this.index)')) \
            .height(js('y.range().band')) \
          .add(pv.Bar) \
            .data(js('function(d) d')) \
            .top(js('function() this.index * y.range().band / m')) \
            .height(js('y.range().band / m')) \
            .left(0) \
            .width(js('x')) \
            .fillStyle(js('pv.Colors.category20().by(pv.index)'))

        # The value label.
        bar.anchor("right").add(pv.Label) \
            .textStyle("white") \
            .text(js('function(d) d.toFixed(1)'))

        # The variable label.
        bar._parent.anchor("left").add(pv.Label) \
            .textAlign("right") \
            .textMargin(5) \
            .text(js('function() "ABCDEFGHIJK".charAt(this.parent.index)'))

        # X-axis ticks.
        self.add(pv.Rule) \
            .data(js('x.ticks(5)')) \
            .left(js('x')) \
            .strokeStyle(
                js('function(d) d ? "rgba(255,255,255,.3)" : "#000"')
            )\
            .add(pv.Rule) \
            .bottom(0) \
            .height(5) \
            .strokeStyle("#000") \
          .anchor("bottom").add(pv.Label) \
            .text(js('x.tickFormat'))




import tw2.core as twc
from tw2.protovis.core.defaults import __basename__, __version__
modname = ".".join(__name__.split('.')[:-1])

pv_js = twc.JSLink(
    modname=modname, filename='static/%s/%s/protovis.js' % (
        __basename__, __version__))

